;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x83e56929013b04696bcde5f3502cabc9 . "./code.cg2js/cg-acache.fasl")
(#x0f8328e21e395d59384759a32fc5cf14 . "./code.cg2js/cg-base.fasl")
(#x8aba23e5290416795c981b9cef194ece . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x2f4f663de2aedd4d7a397d81cb6ac8d5 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x113af705cb40080b358eca3681e52f38 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x275f7aebe8f6353e03f491a57228b6f7 . "./code.cg2js/cg-button.fasl")
(#xa7f429cd35d91fae27d9fb547630c6a0 . "./code.cg2js/cg-calendar.fasl")
(#x50f1dd543aa18035b392984fdceda847 . "./code.cg2js/cg-caret.fasl")
(#x751da3a308f3b4babbbe4d0c9a54409d . "./code.cg2js/cg-chart-or-plot.fasl")
(#xfa99419a9219dc7fe51d70fa63a8eff6 . "./code.cg2js/cg-chart-widget.fasl")
(#xd56fef04b2e71f5c3e91ff2313f5e707 . "./code.cg2js/cg-check-box.fasl")
(#x0a915e9584c7afbe831d9097082cf55b . "./code.cg2js/cg-choice-list.fasl")
(#x3df459956a48f50ace9d86af7c21d892 . "./code.cg2js/cg-choose-printer.fasl")
(#x2cca6128a59dc71df475d9a6256d859c . "./code.cg2js/cg-class-grid.fasl")
(#x4e04a73798c36f17bd93905ada29c67d . "./code.cg2js/cg-class-slot-grid.fasl")
(#xe27904732941c4ed42bf865aed2d1023 . "./code.cg2js/cg-class-support.fasl")
(#x21b2feadae3299bce3c475a495143cb4 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x791b745302c50a95e2a1140b6e558718 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x46e2372dac6d809dc1d5bfd86f9b8bba . "./code.cg2js/cg-clipboard.fasl")
(#xfd8a65927d71a637520213db72b01190 . "./code.cg2js/cg-color-dialog.fasl")
(#x67a1ddd1ebafe1beb1c2f2e14ea7d939 . "./code.cg2js/cg-combo-box.fasl")
(#x6d877fbb1c71a5e7265fe69141f4f2dc . "./code.cg2js/cg-common-control.fasl")
(#x9480758ddbd90bbcefb764b3f715c7ae . "./code.cg2js/cg-comtab.fasl")
(#x69700cb4c1e47bdf33cea4f534ce6cb4 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd179704fbc9f903171541cf020efd055 . "./code.cg2js/cg-curve.fasl")
(#x39ac0f001bd28dcdf3d15679e98a7206 . "./code.cg2js/cg-dde-utils.fasl")
(#xaffee4807a3c6aae8a514e1e8a5d07a6 . "./code.cg2js/cg-dev.fasl")
(#x0cdc81f4b995cc109ac25b95fc36d8e5 . "./code.cg2js/cg-dialog-item.fasl")
(#xadc70e14daf09129a45cbafad27e7045 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xdfa2f6c285b0cc2775518357acca783b . "./code.cg2js/cg-directory-dialog.fasl")
(#x9ee5ffcb0540141fcd8b264d790d9c40 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc1a4316d3aacd65d277c36ff193a4020 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4c45a9d70e14b477c4cf84ae9fe70470 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xe986cc86f6da4eff4f4d4a9455c3136e . "./code.cg2js/cg-drawable.fasl")
(#xb03917fc8126f54b968bbb873ecade1d . "./code.cg2js/cg-dropping-outline.fasl")
(#x284a550ebd00462163b151037b736fe4 . "./code.cg2js/cg-edit-in-place.fasl")
(#x99a634acdaf73678f4e54f7b4fac415b . "./code.cg2js/cg-editable-text.fasl")
(#x236a6637bd243a1a204bd173a937caf8 . "./code.cg2js/cg-file-dialog.fasl")
(#x4ef682fe8e46267d0ccd44fd07ef7267 . "./code.cg2js/cg-fill-texture.fasl")
(#xdaa7341be7773869b8dd842476ece674 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xaf08ffe782859439c6f5991fd4ad575e . "./code.cg2js/cg-font-dialog.fasl")
(#xb8680b3a847304f41b2a1c6741449cfd . "./code.cg2js/cg-gesture-emulation.fasl")
(#x0700db9eed2585650e90debf7c2213e4 . "./code.cg2js/cg-get-pixmap.fasl")
(#x03b0c07d53e435049849a2e6d691e0e6 . "./code.cg2js/cg-get-position.fasl")
(#x94cb6aaf134af9e5f130eefc4047f2e5 . "./code.cg2js/cg-graphics-context.fasl")
(#x89cd1d7d4b1ecd0100a8d6c52937187f . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3a272ed2bb3a6b2845627aa636d5201d . "./code.cg2js/cg-grid-widget.fasl")
(#x92b0a0e449b5d313c46ff6c9fde321fa . "./code.cg2js/cg-group-box.fasl")
(#xb60b85391daf199e6d8339740bb7dba5 . "./code.cg2js/cg-header-control.fasl")
(#xb7a75b1c4cafafcd3eef2f333d9dff1d . "./code.cg2js/cg-hotspot.fasl")
(#xa19fcbf73fe564a267a54c1fd34f2690 . "./code.cg2js/cg-html-dialog.fasl")
(#x211f116cbbc1eab57886b191fabba80c . "./code.cg2js/cg-html-widget.fasl")
(#xecf1aeaac06cacc288399a9a50e53daf . "./code.cg2js/cg-icon-pixmap.fasl")
(#x620d18c157f9f1409a58d349f1bba1ee . "./code.cg2js/cg-icon.fasl")
(#x5dbca9b053361da3bdcb8dd734e00034 . "./code.cg2js/cg-ie.fasl")
(#x4d6dece6b2a3530ef244b2a2dd458650 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe14dfae99c0e2647e9e9c01fe376a5df . "./code.cg2js/cg-intersect.fasl")
(#xe06adbe8b78e17084b4e8519272c68af . "./code.cg2js/cg-item-list.fasl")
(#x8a45b083dfba9f3f1c00649df910cb64 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2c227ed16d55b9d24705c7f0748e270c . "./code.cg2js/cg-lamp.fasl")
(#x5d20b1c2f5b1d79a001c0928cb0fe65b . "./code.cg2js/cg-layout.fasl")
(#xa6ccc28d1bb1edad1ab4ffd09b574f3d . "./code.cg2js/cg-lettered-menu.fasl")
(#xa2f91f2ce153c6528f43478bc8a8dc53 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9f90e2e261a5b32975b365c44ec6995b . "./code.cg2js/cg-lisp-text.fasl")
(#xc22652e712e270c197aea251237fb0b7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x1262399336f73a7c38c55f321211b12a . "./code.cg2js/cg-list-view.fasl")
(#x195958b64d62372a2707c8dde67ef205 . "./code.cg2js/cg-mci.fasl")
(#xaf1bcdb8f3a3e43120ec8bb1b2454958 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x364ae013fa9a0e88cd1dd73da7378dde . "./code.cg2js/cg-menu.fasl")
(#x11e92cbe8b3d5ea5f2b0ee64645cc002 . "./code.cg2js/cg-message-dialog.fasl")
(#x2be140ef0e574390a86a9dd73a1e6c2f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x8dc0ed58216b990d58e41827c2e84c1c . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x01b0d23113da0d8869b63ab2dedc4cc1 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf32ee91e70b90b4dcb41f5f8dc628519 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9366732f0023358b522ffc4140cf446b . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe79eb506ce8285e652c42d5bac990343 . "./code.cg2js/cg-nodes.fasl")
(#xef465d645d2ea90c4fc2f82168170e34 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x23ffff2c25c9dd435e1a23c4a69f3457 . "./code.cg2js/cg-object-editor.fasl")
(#xf89fc6537704a32d98f8d0871867480b . "./code.cg2js/cg-ocx.fasl")
(#x454c2f0c6c26ad55a10d33ec5aef9245 . "./code.cg2js/cg-os-widget.fasl")
(#x0702e861538daf7784835293e4f769f3 . "./code.cg2js/cg-os-window.fasl")
(#xab57b00b2b624ce013dd7c095c5a9cf4 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x48961336642742f489ff1d7c8365ab56 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xa6d0be82eac96d73118aaaa744e55fdb . "./code.cg2js/cg-outline.fasl")
(#x1f25694201b0fa8af94509989056e588 . "./code.cg2js/cg-palette.fasl")
(#x25748cc8810c73482673d683b685680c . "./code.cg2js/cg-paren-matching.fasl")
(#xc03b9c703dcf55c03baaccd1ec7a4899 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x19fa57f305cff958f7ea85ed215b8a8d . "./code.cg2js/cg-picture-widget.fasl")
(#xefc907f65b1e47264260bb1bbe1e09b8 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x124ea93eec06d3bdeffdd905babe013d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8a7650c3d6c091c2b66e08a5c5bb1942 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5cc0d07b8fbf675ca2547cb48d806b07 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x605b56d843261052f092493c74d40316 . "./code.cg2js/cg-pixmap.fasl")
(#x7389b0b50029bdec48e4a2067a572fcd . "./code.cg2js/cg-player.fasl")
(#x6a6b293834d8aaf83788f8c3705494a9 . "./code.cg2js/cg-plot-widget.fasl")
(#x8e2aabb7030a202e93e17aeb928b82ad . "./code.cg2js/cg-printing.fasl")
(#xb6f2030ff8d233b5b0ee2d8828708c22 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd40aff26e9e7acdb016ea76d517ff922 . "./code.cg2js/cg-project-window.fasl")
(#xb938eec85a8c3ffeab80a4864fb14dac . "./code.cg2js/cg-property.fasl")
(#x64c7e81ef68246500ff17fd91b998b0e . "./code.cg2js/cg-radio-button.fasl")
(#x9a4496f0abbfa544c73faf99ca3da867 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x6514b29a306522f9498db975b926d038 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xb35e8fd3ca3eefed96ed9a433e25a584 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xd186b40965c336e6582e8c8574e9765e . "./code.cg2js/cg-rich-edit.fasl")
(#x586f96c96b76363063d0f52a47a06d22 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x88aaccdace0e9be792ba611946925577 . "./code.cg2js/cg-scaling-stream.fasl")
(#x8fe0fbb4c74e0dba517ec3d29cdd5108 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe673eb16aa315c1ed0e73f6e002a64fc . "./code.cg2js/cg-scroll-bar.fasl")
(#xe14c711e84188c9766065dc88c5822f4 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6e196ca3e0eddb4491f5cfde6da69884 . "./code.cg2js/cg-selected-object.fasl")
(#x235a98817a86f2b7bab46f4b9e75b0d7 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x97cc6f9710c8601301f15b3d4a6b3eec . "./code.cg2js/cg-split-bar.fasl")
(#x87693e7767561b07d0c41e0d7348fc6c . "./code.cg2js/cg-static-text.fasl")
(#x3a8764e439e161d4831b5dcd2acbd8b8 . "./code.cg2js/cg-status-bar.fasl")
(#x49b0ae1be5733aaf2780b9943382d753 . "./code.cg2js/cg-string-dialog.fasl")
(#x7cadbeaadc273f6a01ec3f119bda87f9 . "./code.cg2js/cg-tab-control.fasl")
(#xf935138086d6b7ea044cc15a4a86a68f . "./code.cg2js/cg-template-string.fasl")
(#xfde38c411912cf06f72d4de0c4958c3b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x5ac5495e467a7f530d3e722bbcfe08b6 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x99948a0541cf3fde5feffec468fe50d3 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1400ba0b5b419b1873c110a28cd8032a . "./code.cg2js/cg-text-or-combo.fasl")
(#x9b970979043a896c9a8660d7d5f99724 . "./code.cg2js/cg-text-widget.fasl")
(#x0730daf9c9cbb2f8ab148ce92bcdaf0e . "./code.cg2js/cg-timer.fasl")
(#xd4a1b9d7d3a696065a34b31e86d273b4 . "./code.cg2js/cg-toggling-widget.fasl")
(#xc6f4e75ea3a1b3dab021afaeca16a516 . "./code.cg2js/cg-toolbar.fasl")
(#xecba3a7c1088acf9b678f8d33ed04678 . "./code.cg2js/cg-tooltip.fasl")
(#x947046638f3400e04bc6e37e4fc773dd . "./code.cg2js/cg-touch.fasl")
(#x5821f281df31fb4aef59c232aeaa0813 . "./code.cg2js/cg-trackbar.fasl")
(#x3f1a9777dac1cada34210a763ae9dfa0 . "./code.cg2js/cg-tray.fasl")
(#xcb9084da9e30292b473fee97f7910f62 . "./code.cg2js/cg-up-down-control.fasl")
(#x210ecfda2ecb30b3e0f40634a805a459 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe6f0692b2818c6bc768a47220b68e0ec . "./code.cg2js/cg-web-browser-dde.fasl")
(#x5aa46e316d800bce75c186cc129454bc . "./code.cg2js/cg-web-browser.fasl")
(#xc61903891cbb6b6eb639d038a512872e . "./code.cg2js/cg-wrap-string.fasl")
(#x177e48f84b668f4ca3fc9d84193b31b5 . "./code.cg2js/cg-yes-no-list.fasl")
(#xba920f9b8d13f3e21b8dba5c724ebf63 . "./code.cg2js/cg.fasl")
(#xd8f868b7d57e5118f1ddf6344af15ea8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x86b99416f95f005b44dfa8112c50739a . "./code.idejs/ide-acache.fasl")
(#x611cf559116c52ebfe366816f24979d4 . "./code.idejs/ide-apropos.fasl")
(#x3370efe2fc8ca66622605b7e783288ca . "./code.idejs/ide-base.fasl")
(#x86d4f22b577b7293a8d6ee5c0f47f2af . "./code.idejs/ide-class-browser.fasl")
(#xc7caa9df0cce003d2e92d100331d63c5 . "./code.idejs/ide-class-grapher.fasl")
(#x425e912f73a9733b5353d8bc3f5d19ba . "./code.idejs/ide-clipboard.fasl")
(#xea884edf34daa76b5cec556b4924f262 . "./code.idejs/ide-dev.fasl")
(#x2551aa9ab1d0a51627ca421c7d7d9204 . "./code.idejs/ide-editor.fasl")
(#xa11b0febc92c476c8d849febce88f07d . "./code.idejs/ide-find-definitions.fasl")
(#xdf2be8113ac82f592d97a15988d70ee5 . "./code.idejs/ide-find-in-files.fasl")
(#x06d1a343741d8297322a49fb3fdacc4d . "./code.idejs/ide-form.fasl")
(#x93d5e4e233ef9907800880934f6695f8 . "./code.idejs/ide-grapher.fasl")
(#x1c579ee0f9a2e7b34e188b013b98be06 . "./code.idejs/ide-inspector.fasl")
(#xb3d4f93496b013114e920e863dd01bac . "./code.idejs/ide-intro.fasl")
(#xe0fca150a07edb83c9e70f8bbc21d622 . "./code.idejs/ide-keyshort.fasl")
(#x5bac3856bd59808458935de565ce448c . "./code.idejs/ide-navigator.fasl")
(#x02a90b7bd33dfc0cf3fe3f6727807692 . "./code.idejs/ide-package-list.fasl")
(#x3071fe7f1b2f8f2fbcb997f9461fb15d . "./code.idejs/ide-patch-dialog.fasl")
(#xdf92570b801a475fa7754bf4d060b6ce . "./code.idejs/ide-process-browser.fasl")
(#xf46e981956be8cc2355c3938de362526 . "./code.idejs/ide-project.fasl")
(#x754deb54920f2e35936823678709e806 . "./code.idejs/ide-trace.fasl")
(#x5c87d68a192b16901e74f504cd2168e1 . "./code.idejs/ide-window-list.fasl")
(#x63d43715fc24dfcc27ef49e2e4a3653a . "./code.idejs/ide.fasl")
(#x3d70f7088b7ddd139f75c06fecd8ac58 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
